@@ -23,7 +23,7 @@ namespace redis {
 std::vector<ReplyUPtr> PipelineImpl::exec(Connection &connection, std::size_t cmd_num) {
     std::vector<ReplyUPtr> replies;
     while (cmd_num > 0) {
-        replies.push_back(connection.recv(false));
+        replies.push_back(connection.recv());
         --cmd_num;
     }
 
