@@ -4,7 +4,7 @@
  * Use of this software is governed by the Business Source License included
  * in the LICENSE.TXT file in the project's root directory.
  *
- * Change Date: 2023-01-01
+ * Change Date: 2026-01-01
  *
  * On the date above, in accordance with the Business Source License, use
  * of this software will be governed by version 2.0 of the Apache License.
@@ -55,10 +55,14 @@ public:
 	{
 		for(unsigned int i=0;i<C;++i) {
 			if ((s)&&(i < len)) {
-				if (!(_d[i] = *s))
+				if (!(_d[i] = *s)) {
 					s = (const char *)0;
-				else ++s;
-			} else _d[i] = (char)0;
+				} else {
+					++s;
+				}
+			} else {
+				_d[i] = (char)0;
+			}
 		}
 		_d[C - 1] = (char)0;
 	}
@@ -82,10 +86,14 @@ public:
 	{
 		for(unsigned int i=0;i<C;++i) {
 			if (s) {
-				if (!(_d[i] = *s))
+				if (!(_d[i] = *s)) {
 					s = (const char *)0;
-				else ++s;
-			} else _d[i] = (char)0;
+				} else {
+					++s;
+				}
+			} else {
+				_d[i] = (char)0;
+			}
 		}
 		_d[C - 1] = (char)0;
 		return (!s);
@@ -105,8 +113,9 @@ public:
 	inline unsigned int sizeBytes() const
 	{
 		for(unsigned int i=0;i<C;++i) {
-			if (!_d[i])
+			if (!_d[i]) {
 				return i;
+			}
 		}
 		return C-1;
 	}
@@ -141,14 +150,16 @@ public:
 		bool esc;
 		int j;
 
-		if (!destlen) // sanity check
+		if (!destlen) { // sanity check
 			return -1;
+		}
 
 		while (*p) {
 			k = key;
 			while ((*k)&&(*p)) {
-				if (*p != *k)
+				if (*p != *k) {
 					break;
+				}
 				++k;
 				if (++p == eof) {
 					dest[0] = (char)0;
@@ -164,11 +175,21 @@ public:
 					if (esc) {
 						esc = false;
 						switch(*p) {
-							case 'r': dest[j++] = 13; break;
-							case 'n': dest[j++] = 10; break;
-							case '0': dest[j++] = (char)0; break;
-							case 'e': dest[j++] = '='; break;
-							default: dest[j++] = *p; break;
+							case 'r':
+								dest[j++] = 13;
+								break;
+							case 'n':
+								dest[j++] = 10;
+								break;
+							case '0':
+								dest[j++] = (char)0;
+								break;
+							case 'e':
+								dest[j++] = '=';
+								break;
+							default:
+								dest[j++] = *p;
+								break;
 						}
 						if (j == (int)destlen) {
 							dest[j-1] = (char)0;
@@ -202,8 +223,9 @@ public:
 						dest[0] = (char)0;
 						return -1;
 					}
+				} else {
+					break;
 				}
-				else break;
 			}
 		}
 
@@ -242,8 +264,9 @@ public:
 	bool getB(const char *key,bool dfl = false) const
 	{
 		char tmp[4];
-		if (this->get(key,tmp,sizeof(tmp)) >= 0)
+		if (this->get(key,tmp,sizeof(tmp)) >= 0) {
 			return ((*tmp == '1')||(*tmp == 't')||(*tmp == 'T'));
+		}
 		return dfl;
 	}
 
@@ -257,8 +280,9 @@ public:
 	inline uint64_t getUI(const char *key,uint64_t dfl = 0) const
 	{
 		char tmp[128];
-		if (this->get(key,tmp,sizeof(tmp)) >= 1)
+		if (this->get(key,tmp,sizeof(tmp)) >= 1) {
 			return Utils::hexStrToU64(tmp);
+		}
 		return dfl;
 	}
 
@@ -272,8 +296,9 @@ public:
 	inline int64_t getI(const char *key,int64_t dfl = 0) const
 	{
 		char tmp[128];
-		if (this->get(key,tmp,sizeof(tmp)) >= 1)
+		if (this->get(key,tmp,sizeof(tmp)) >= 1) {
 			return Utils::hexStrTo64(tmp);
+		}
 		return dfl;
 	}
 
@@ -335,11 +360,21 @@ public:
 								return false;
 							}
 							switch(*p) {
-								case 0: _d[j++] = '0'; break;
-								case 13: _d[j++] = 'r'; break;
-								case 10: _d[j++] = 'n'; break;
-								case '\\': _d[j++] = '\\'; break;
-								case '=': _d[j++] = 'e'; break;
+								case 0:
+									_d[j++] = '0';
+									break;
+								case 13:
+									_d[j++] = 'r';
+									break;
+								case 10:
+									_d[j++] = 'n';
+									break;
+								case '\\':
+									_d[j++] = '\\';
+									break;
+								case '=':
+									_d[j++] = 'e';
+									break;
 							}
 							if (j == C) {
 								_d[i] = (char)0;
