@@ -4,7 +4,7 @@
  * Use of this software is governed by the Business Source License included
  * in the LICENSE.TXT file in the project's root directory.
  *
- * Change Date: 2023-01-01
+ * Change Date: 2026-01-01
  *
  * On the date above, in accordance with the Business Source License, use
  * of this software will be governed by version 2.0 of the Apache License.
@@ -34,7 +34,7 @@ namespace ZeroTier {
 static inline void _computeMemoryHardHash(const void *publicKey,unsigned int publicKeyBytes,void *digest,void *genmem)
 {
 	// Digest publicKey[] to obtain initial digest
-	SHA512::hash(digest,publicKey,publicKeyBytes);
+	SHA512(digest,publicKey,publicKeyBytes);
 
 	// Initialize genmem[] using Salsa20 in a CBC-like configuration since
 	// ordinary Salsa20 is randomly seek-able. This is good for a cipher
@@ -93,8 +93,9 @@ void Identity::generate()
 	} while (_address.isReserved());
 
 	_publicKey = kp.pub;
-	if (!_privateKey)
+	if (!_privateKey) {
 		_privateKey = new C25519::Private();
+	}
 	*_privateKey = kp.priv;
 
 	delete [] genmem;
@@ -102,8 +103,9 @@ void Identity::generate()
 
 bool Identity::locallyValidate() const
 {
-	if (_address.isReserved())
+	if (_address.isReserved()) {
 		return false;
+	}
 
 	unsigned char digest[64];
 	char *genmem = new char[ZT_IDENTITY_GEN_MEMORY];
