@@ -4,7 +4,7 @@
  * Use of this software is governed by the Business Source License included
  * in the LICENSE.TXT file in the project's root directory.
  *
- * Change Date: 2023-01-01
+ * Change Date: 2026-01-01
  *
  * On the date above, in accordance with the Business Source License, use
  * of this software will be governed by version 2.0 of the Apache License.
@@ -33,38 +33,80 @@ InetAddress::IpScope InetAddress::ipScope() const
 		case AF_INET: {
 			const uint32_t ip = Utils::ntoh((uint32_t)reinterpret_cast<const struct sockaddr_in *>(this)->sin_addr.s_addr);
 			switch(ip >> 24) {
-				case 0x00: return IP_SCOPE_NONE;                                   // 0.0.0.0/8 (reserved, never used)
-				case 0x06: return IP_SCOPE_PSEUDOPRIVATE;                          // 6.0.0.0/8 (US Army)
-				case 0x0a: return IP_SCOPE_PRIVATE;                                // 10.0.0.0/8
-				case 0x0b: return IP_SCOPE_PSEUDOPRIVATE;                          // 11.0.0.0/8 (US DoD)
-				case 0x15: return IP_SCOPE_PSEUDOPRIVATE;                          // 21.0.0.0/8 (US DDN-RVN)
-				case 0x16: return IP_SCOPE_PSEUDOPRIVATE;                          // 22.0.0.0/8 (US DISA)
-				case 0x19: return IP_SCOPE_PSEUDOPRIVATE;                          // 25.0.0.0/8 (UK Ministry of Defense)
-				case 0x1a: return IP_SCOPE_PSEUDOPRIVATE;                          // 26.0.0.0/8 (US DISA)
-				case 0x1c: return IP_SCOPE_PSEUDOPRIVATE;                          // 28.0.0.0/8 (US DSI-North)
-				case 0x1d: return IP_SCOPE_PSEUDOPRIVATE;                          // 29.0.0.0/8 (US DISA)
-				case 0x1e: return IP_SCOPE_PSEUDOPRIVATE;                          // 30.0.0.0/8 (US DISA)
-				case 0x33: return IP_SCOPE_PSEUDOPRIVATE;                          // 51.0.0.0/8 (UK Department of Social Security)
-				case 0x37: return IP_SCOPE_PSEUDOPRIVATE;                          // 55.0.0.0/8 (US DoD)
-				case 0x38: return IP_SCOPE_PSEUDOPRIVATE;                          // 56.0.0.0/8 (US Postal Service)
+				case 0x00:
+					return IP_SCOPE_NONE;           // 0.0.0.0/8 (reserved, never used)
+				case 0x06:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 6.0.0.0/8 (US Army)
+				case 0x0a:
+					return IP_SCOPE_PRIVATE;        // 10.0.0.0/8
+				case 0x0b:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 11.0.0.0/8 (US DoD)
+				case 0x15:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 21.0.0.0/8 (US DDN-RVN)
+				case 0x16:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 22.0.0.0/8 (US DISA)
+				case 0x19:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 25.0.0.0/8 (UK Ministry of Defense)
+				case 0x1a:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 26.0.0.0/8 (US DISA)
+				case 0x1c:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 28.0.0.0/8 (US DSI-North)
+				case 0x1d:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 29.0.0.0/8 (US DISA)
+				case 0x1e:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 30.0.0.0/8 (US DISA)
+				case 0x33:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 51.0.0.0/8 (UK Department of Social Security)
+				case 0x37:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 55.0.0.0/8 (US DoD)
+				case 0x38:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 56.0.0.0/8 (US Postal Service)
 				case 0x64:
-					if ((ip & 0xffc00000) == 0x64400000) return IP_SCOPE_PRIVATE;    // 100.64.0.0/10
+					if ((ip & 0xffc00000) == 0x64400000) {
+						return IP_SCOPE_PRIVATE;    // 100.64.0.0/10
+					}
 					break;
-				case 0x7f: return IP_SCOPE_LOOPBACK;                               // 127.0.0.0/8
+				case 0x7f:
+					return IP_SCOPE_LOOPBACK;       // 127.0.0.0/8
 				case 0xa9:
-					if ((ip & 0xffff0000) == 0xa9fe0000) return IP_SCOPE_LINK_LOCAL; // 169.254.0.0/16
+					if ((ip & 0xffff0000) == 0xa9fe0000) {
+					 	return IP_SCOPE_LINK_LOCAL; // 169.254.0.0/16
+					}
 					break;
 				case 0xac:
-					if ((ip & 0xfff00000) == 0xac100000) return IP_SCOPE_PRIVATE;    // 172.16.0.0/12
+					if ((ip & 0xfff00000) == 0xac100000) {
+						return IP_SCOPE_PRIVATE;    // 172.16.0.0/12
+					}
 					break;
 				case 0xc0:
-					if ((ip & 0xffff0000) == 0xc0a80000) return IP_SCOPE_PRIVATE;    // 192.168.0.0/16
+					if ((ip & 0xffff0000) == 0xc0a80000) {
+						return IP_SCOPE_PRIVATE;    // 192.168.0.0/16
+					}
+					if ((ip & 0xffffff00) == 0xc0000200) {
+						return IP_SCOPE_PRIVATE;    // 192.0.2.0/24
+					}
 					break;
-				case 0xff: return IP_SCOPE_NONE;                                   // 255.0.0.0/8 (broadcast, or unused/unusable)
+				case 0xc6:
+					if ((ip & 0xfffe0000) == 0xc6120000) {
+						return IP_SCOPE_PRIVATE;    // 198.18.0.0/15
+					}
+					if ((ip & 0xffffff00) == 0xc6336400) {
+						return IP_SCOPE_PRIVATE;    // 198.51.100.0/24
+					}
+					break;
+				case 0xcb:
+					if ((ip & 0xffffff00) == 0xcb007100) {
+						return IP_SCOPE_PRIVATE;    // 203.0.113.0/24
+					}
+					break;
+				case 0xff:
+					return IP_SCOPE_NONE;           // 255.0.0.0/8 (broadcast, or unused/unusable)
 			}
 			switch(ip >> 28) {
-				case 0xe: return IP_SCOPE_MULTICAST;                               // 224.0.0.0/4
-				case 0xf: return IP_SCOPE_PSEUDOPRIVATE;                           // 240.0.0.0/4 ("reserved," usually unusable)
+				case 0xe:
+					return IP_SCOPE_MULTICAST;      // 224.0.0.0/4
+				case 0xf:
+					return IP_SCOPE_PSEUDOPRIVATE;  // 240.0.0.0/4 ("reserved," usually unusable)
 			}
 			return IP_SCOPE_GLOBAL;
 		}	break;
@@ -72,21 +114,48 @@ InetAddress::IpScope InetAddress::ipScope() const
 		case AF_INET6: {
 			const unsigned char *ip = reinterpret_cast<const unsigned char *>(reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_addr.s6_addr);
 			if ((ip[0] & 0xf0) == 0xf0) {
-				if (ip[0] == 0xff) return IP_SCOPE_MULTICAST;                      // ff00::/8
+				if (ip[0] == 0xff) {
+					return IP_SCOPE_MULTICAST;      // ff00::/8
+				}
 				if ((ip[0] == 0xfe)&&((ip[1] & 0xc0) == 0x80)) {
 					unsigned int k = 2;
-					while ((!ip[k])&&(k < 15)) ++k;
-					if ((k == 15)&&(ip[15] == 0x01))
-						return IP_SCOPE_LOOPBACK;                                      // fe80::1/128
-					else return IP_SCOPE_LINK_LOCAL;                                 // fe80::/10
+					while ((!ip[k])&&(k < 15)) {
+						++k;
+					}
+					if ((k == 15)&&(ip[15] == 0x01)) {
+						return IP_SCOPE_LOOPBACK;   // fe80::1/128
+					} else {
+						return IP_SCOPE_LINK_LOCAL; // fe80::/10
+					}
+				}
+				if ((ip[0] & 0xfe) == 0xfc) {
+					return IP_SCOPE_PRIVATE;        // fc00::/7
 				}
-				if ((ip[0] & 0xfe) == 0xfc) return IP_SCOPE_PRIVATE;               // fc00::/7
 			}
+
+			// :::ffff:127.0.0.1
+			// 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0xff, 0xff, 0x7f, 0, 0, 1
 			unsigned int k = 0;
-			while ((!ip[k])&&(k < 15)) ++k;
+			while ((!ip[k])&&(k < 9)) {
+				++k;
+			}
+			if (k == 9) {
+				if (ip[10] == 0xff && ip[11] == 0xff && ip[12] == 0x7f) {
+					return IP_SCOPE_LOOPBACK;
+				}
+			}
+
+			k = 0;
+			while ((!ip[k])&&(k < 15)) {
+				++k;
+			}
 			if (k == 15) { // all 0's except last byte
-				if (ip[15] == 0x01) return IP_SCOPE_LOOPBACK;                      // ::1/128
-				if (ip[15] == 0x00) return IP_SCOPE_NONE;                          // ::/128
+				if (ip[15] == 0x01) {
+					return IP_SCOPE_LOOPBACK;       // ::1/128
+				}
+				if (ip[15] == 0x00) {
+					return IP_SCOPE_NONE;           // ::/128
+				}
 			}
 			return IP_SCOPE_GLOBAL;
 		}	break;
@@ -116,7 +185,9 @@ char *InetAddress::toString(char buf[64]) const
 {
 	char *p = toIpString(buf);
 	if (*p) {
-		while (*p) ++p;
+		while (*p) {
+			++p;
+		}
 		*(p++) = '/';
 		Utils::decimal(port(),p);
 	}
@@ -152,14 +223,17 @@ bool InetAddress::fromString(const char *ipSlashPort)
 
 	memset(this,0,sizeof(InetAddress));
 
-	if (!*ipSlashPort)
+	if (!*ipSlashPort) {
 		return true;
-	if (!Utils::scopy(buf,sizeof(buf),ipSlashPort))
+	}
+	if (!Utils::scopy(buf,sizeof(buf),ipSlashPort)) {
 		return false;
+	}
 
 	char *portAt = buf;
-	while ((*portAt)&&(*portAt != '/'))
+	while ((*portAt)&&(*portAt != '/')) {
 		++portAt;
+	}
 	unsigned int port = 0;
 	if (*portAt) {
 		*(portAt++) = (char)0;
@@ -247,8 +321,9 @@ bool InetAddress::isEqualPrefix(const InetAddress &addr) const
 				const uint8_t *a = reinterpret_cast<const uint8_t *>(reinterpret_cast<const struct sockaddr_in6 *>(&addr)->sin6_addr.s6_addr);
 				const uint8_t *b = reinterpret_cast<const uint8_t *>(reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_addr.s6_addr);
 				for(unsigned int i=0;i<16;++i) {
-					if ((a[i] & m[i]) != (b[i] & n[i]))
+					if ((a[i] & m[i]) != (b[i] & n[i])) {
 						return false;
+					}
 				}
 				return true;
 			}
@@ -263,8 +338,9 @@ bool InetAddress::containsAddress(const InetAddress &addr) const
 		switch(ss_family) {
 			case AF_INET: {
 				const unsigned int bits = netmaskBits();
-				if (bits == 0)
+				if (bits == 0) {
 					return true;
+				}
 				return ( (Utils::ntoh((uint32_t)reinterpret_cast<const struct sockaddr_in *>(&addr)->sin_addr.s_addr) >> (32 - bits)) == (Utils::ntoh((uint32_t)reinterpret_cast<const struct sockaddr_in *>(this)->sin_addr.s_addr) >> (32 - bits)) );
 			}
 			case AF_INET6: {
@@ -273,8 +349,9 @@ bool InetAddress::containsAddress(const InetAddress &addr) const
 				const uint8_t *a = reinterpret_cast<const uint8_t *>(reinterpret_cast<const struct sockaddr_in6 *>(&addr)->sin6_addr.s6_addr);
 				const uint8_t *b = reinterpret_cast<const uint8_t *>(reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_addr.s6_addr);
 				for(unsigned int i=0;i<16;++i) {
-					if ((a[i] & m[i]) != b[i])
+					if ((a[i] & m[i]) != b[i]) {
 						return false;
+					}
 				}
 				return true;
 			}
@@ -288,26 +365,32 @@ bool InetAddress::isNetwork() const
 	switch(ss_family) {
 		case AF_INET: {
 			unsigned int bits = netmaskBits();
-			if (bits <= 0)
+			if (bits <= 0) {
 				return false;
-			if (bits >= 32)
+			}
+			if (bits >= 32) {
 				return false;
+			}
 			uint32_t ip = Utils::ntoh((uint32_t)reinterpret_cast<const struct sockaddr_in *>(this)->sin_addr.s_addr);
 			return ((ip & (0xffffffff >> bits)) == 0);
 		}
 		case AF_INET6: {
 			unsigned int bits = netmaskBits();
-			if (bits <= 0)
+			if (bits <= 0) {
 				return false;
-			if (bits >= 128)
+			}
+			if (bits >= 128) {
 				return false;
+			}
 			const unsigned char *ip = reinterpret_cast<const unsigned char *>(reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_addr.s6_addr);
 			unsigned int p = bits / 8;
-			if ((ip[p++] & (0xff >> (bits % 8))) != 0)
+			if ((ip[p++] & (0xff >> (bits % 8))) != 0) {
 				return false;
+			}
 			while (p < 16) {
-				if (ip[p++])
+				if (ip[p++]) {
 					return false;
+				}
 			}
 			return true;
 		}
@@ -340,30 +423,32 @@ bool InetAddress::operator==(const InetAddress &a) const
 
 bool InetAddress::operator<(const InetAddress &a) const
 {
-	if (ss_family < a.ss_family)
+	if (ss_family < a.ss_family) {
 		return true;
-	else if (ss_family == a.ss_family) {
+	} else if (ss_family == a.ss_family) {
 		switch(ss_family) {
 			case AF_INET:
-				if (reinterpret_cast<const struct sockaddr_in *>(this)->sin_port < reinterpret_cast<const struct sockaddr_in *>(&a)->sin_port)
+				if (reinterpret_cast<const struct sockaddr_in *>(this)->sin_port < reinterpret_cast<const struct sockaddr_in *>(&a)->sin_port) {
 					return true;
-				else if (reinterpret_cast<const struct sockaddr_in *>(this)->sin_port == reinterpret_cast<const struct sockaddr_in *>(&a)->sin_port) {
-					if (reinterpret_cast<const struct sockaddr_in *>(this)->sin_addr.s_addr < reinterpret_cast<const struct sockaddr_in *>(&a)->sin_addr.s_addr)
+				} else if (reinterpret_cast<const struct sockaddr_in *>(this)->sin_port == reinterpret_cast<const struct sockaddr_in *>(&a)->sin_port) {
+					if (reinterpret_cast<const struct sockaddr_in *>(this)->sin_addr.s_addr < reinterpret_cast<const struct sockaddr_in *>(&a)->sin_addr.s_addr) {
 						return true;
+					}
 				}
 				break;
 			case AF_INET6:
-				if (reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_port < reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_port)
+				if (reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_port < reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_port) {
 					return true;
-				else if (reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_port == reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_port) {
-					if (reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_flowinfo < reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_flowinfo)
+				} else if (reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_port == reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_port) {
+					if (reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_flowinfo < reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_flowinfo) {
 						return true;
-					else if (reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_flowinfo == reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_flowinfo) {
-						if (memcmp(reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_addr.s6_addr,reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_addr.s6_addr,16) < 0)
+					} else if (reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_flowinfo == reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_flowinfo) {
+						if (memcmp(reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_addr.s6_addr,reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_addr.s6_addr,16) < 0) {
 							return true;
-						else if (memcmp(reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_addr.s6_addr,reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_addr.s6_addr,16) == 0) {
-							if (reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_scope_id < reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_scope_id)
+						} else if (memcmp(reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_addr.s6_addr,reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_addr.s6_addr,16) == 0) {
+							if (reinterpret_cast<const struct sockaddr_in6 *>(this)->sin6_scope_id < reinterpret_cast<const struct sockaddr_in6 *>(&a)->sin6_scope_id) {
 								return true;
+							}
 						}
 					}
 				}
