@@ -4,7 +4,7 @@
  * Use of this software is governed by the Business Source License included
  * in the LICENSE.TXT file in the project's root directory.
  *
- * Change Date: 2025-01-01
+ * Change Date: 2026-01-01
  *
  * On the date above, in accordance with the Business Source License, use
  * of this software will be governed by version 2.0 of the Apache License.
@@ -193,11 +193,9 @@ public:
 	{
 		if (end == begin) {
 			return wrap ? S : 0;
-		}
-		else if (end > begin) {
+		} else if (end > begin) {
 			return end - begin;
-		}
-		else {
+		} else {
 			return S + end - begin;
 		}
 	}
