@@ -4,7 +4,7 @@
  * Use of this software is governed by the Business Source License included
  * in the LICENSE.TXT file in the project's root directory.
  *
- * Change Date: 2023-01-01
+ * Change Date: 2026-01-01
  *
  * On the date above, in accordance with the Business Source License, use
  * of this software will be governed by version 2.0 of the Apache License.
@@ -23,8 +23,9 @@ namespace ZeroTier {
 
 int Tag::verify(const RuntimeEnvironment *RR,void *tPtr) const
 {
-	if ((!_signedBy)||(_signedBy != Network::controllerFor(_networkId)))
+	if ((!_signedBy)||(_signedBy != Network::controllerFor(_networkId))) {
 		return -1;
+	}
 	const Identity id(RR->topology->getIdentity(tPtr,_signedBy));
 	if (!id) {
 		RR->sw->requestWhois(tPtr,RR->node->now(),_signedBy);
