@@ -4,7 +4,7 @@
  * Use of this software is governed by the Business Source License included
  * in the LICENSE.TXT file in the project's root directory.
  *
- * Change Date: 2025-01-01
+ * Change Date: 2026-01-01
  *
  * On the date above, in accordance with the Business Source License, use
  * of this software will be governed by version 2.0 of the Apache License.
@@ -188,18 +188,22 @@ const Utils::CPUIDRegisters Utils::CPUID;
 static void _Utils_doBurn(volatile uint8_t *ptr,unsigned int len)
 {
 	volatile uint8_t *const end = ptr + len;
-	while (ptr != end) *(ptr++) = (uint8_t)0;
+	while (ptr != end) {
+		*(ptr++) = (uint8_t)0;
+	}
 }
 static void (*volatile _Utils_doBurn_ptr)(volatile uint8_t *,unsigned int) = _Utils_doBurn;
 void Utils::burn(void *ptr,unsigned int len) { (_Utils_doBurn_ptr)((volatile uint8_t *)ptr,len); }
 
 static unsigned long _Utils_itoa(unsigned long n,char *s)
 {
-	if (n == 0)
+	if (n == 0) {
 		return 0;
+	}
 	unsigned long pos = _Utils_itoa(n / 10,s);
-	if (pos >= 22) // sanity check, should be impossible
+	if (pos >= 22) { // sanity check, should be impossible
 		pos = 22;
+	}
 	s[pos] = '0' + (char)(n % 10);
 	return pos + 1;
 }
@@ -288,7 +292,9 @@ void Utils::getSecureRandom(void *buf,unsigned int bytes)
 						exit(1);
 						return;
 					}
-				} else break;
+				} else {
+					break;
+				}
 			}
 			randomPtr = 0;
 			s20.crypt12(randomBuf,randomBuf,sizeof(randomBuf));
