@@ -4,7 +4,7 @@
  * Use of this software is governed by the Business Source License included
  * in the LICENSE.TXT file in the project's root directory.
  *
- * Change Date: 2025-01-01
+ * Change Date: 2026-01-01
  *
  * On the date above, in accordance with the Business Source License, use
  * of this software will be governed by version 2.0 of the Apache License.
@@ -171,7 +171,7 @@ static int cli(int argc,char **argv)
 #endif
 {
 	unsigned int port = 0;
-	std::string homeDir,command,arg1,arg2,authToken;
+	std::string homeDir,command,arg1,arg2,arg3,arg4,authToken;
 	std::string ip("127.0.0.1");
 	bool json = false;
 	for(int i=1;i<argc;++i) {
@@ -569,9 +569,35 @@ static int cli(int argc,char **argv)
 				return 1;
 			}
 		}
+		else if (arg1 == "setmtu") { /* zerotier-cli bond setmtu <mtu> <iface> <ip> */
+			requestHeaders["Content-Type"] = "application/json";
+			requestHeaders["Content-Length"] = "2";
+			if (argc == 8) {
+				arg2 = argv[5];
+				arg3 = argv[6];
+				arg4 = argv[7];
+			}
+			unsigned int scode = Http::POST(
+				1024 * 1024 * 16,
+				60000,
+				(const struct sockaddr *)&addr,
+				(std::string("/bond/") + arg1 + "/" + arg2 + "/" + arg3 + "/" + arg4).c_str(),
+				requestHeaders,
+				"{}",
+				2,
+				responseHeaders,
+				responseBody);
+			if (scode == 200) {
+				printf("200 setmtu OK" ZT_EOL_S);
+				return 0;
+			} else {
+				printf("%d Failed to set MTU: %s" ZT_EOL_S, scode, responseBody.c_str());
+				return 1;
+			}
+			return 0;
+		}
 		else if (arg1.length() == 10) {
 			if (arg2 == "rotate") { /* zerotier-cli bond <peerId> rotate */
-				fprintf(stderr, "zerotier-cli bond <peerId> rotate\n");
 				requestHeaders["Content-Type"] = "application/json";
 				requestHeaders["Content-Length"] = "2";
 				unsigned int scode = Http::POST(
@@ -588,7 +614,7 @@ static int cli(int argc,char **argv)
 					if (json) {
 						printf("%s",cliFixJsonCRs(responseBody).c_str());
 					} else {
-						printf("200 bond OK" ZT_EOL_S);
+						printf("200 rotate OK" ZT_EOL_S);
 					}
 					return 0;
 				} else {
@@ -638,37 +664,37 @@ static int cli(int argc,char **argv)
 							printf("\nidx"
 							"                  interface"
 							"                                  "
-							"path               socket\n");
-							for(int i=0; i<100; i++) { printf("-"); }
+							"path               socket             local port\n");
+							for(int i=0; i<120; i++) { printf("-"); }
 							printf("\n");
 							for (int i=0; i<p.size(); i++)
 							{
-								printf("%2d: %26s %51s %.16llx\n",
+								printf("%2d: %26s %51s %.16llx %12d\n",
 									i,
 									OSUtils::jsonString(p[i]["ifname"],"-").c_str(),
 									OSUtils::jsonString(p[i]["address"],"-").c_str(),
-									(unsigned long long)OSUtils::jsonInt(p[i]["localSocket"],0)
+									(unsigned long long)OSUtils::jsonInt(p[i]["localSocket"],0),
+									(uint16_t)OSUtils::jsonInt(p[i]["localPort"],0)
 									);
 							}
-							printf("\nidx     lat      pdv     "
-							"plr     per    capacity    qual      "
-							"rx_age      tx_age  eligible  bonded\n");
-							for(int i=0; i<100; i++) { printf("-"); }
+							printf("\nidx     lat      pdv    "
+							"capacity    qual      "
+							"rx_age      tx_age  eligible  bonded   flows\n");
+							for(int i=0; i<120; i++) { printf("-"); }
 							printf("\n");
 							for (int i=0; i<p.size(); i++)
 							{
-								printf("%2d: %8.2f %8.2f %7.4f %7.4f %10d %7.4f %11d %11d %9d %7d\n",
+								printf("%2d: %8.2f %8.2f %10d %7.4f %11d %11d %9d %7d %7d\n",
 									i,
 									OSUtils::jsonDouble(p[i]["latencyMean"], 0),
 									OSUtils::jsonDouble(p[i]["latencyVariance"], 0),
-									OSUtils::jsonDouble(p[i]["packetLossRatio"], 0),
-									OSUtils::jsonDouble(p[i]["packetErrorRatio"], 0),
 									(int)OSUtils::jsonInt(p[i]["givenLinkSpeed"], 0),
 									OSUtils::jsonDouble(p[i]["relativeQuality"], 0),
 									(int)OSUtils::jsonInt(p[i]["lastInAge"], 0),
 									(int)OSUtils::jsonInt(p[i]["lastOutAge"], 0),
 									(int)OSUtils::jsonInt(p[i]["eligible"],0),
-									(int)OSUtils::jsonInt(p[i]["bonded"],0));
+									(int)OSUtils::jsonInt(p[i]["bonded"],0),
+									(int)OSUtils::jsonInt(p[i]["assignedFlowCount"],0));
 							}
 						}
 					}
@@ -680,6 +706,7 @@ static int cli(int argc,char **argv)
 				return 2;
 			}
 		}
+
 		/* zerotier-cli bond command was malformed in some way */
 		printf("(bond) command is missing required arguments" ZT_EOL_S);
 		return 2;
@@ -1184,7 +1211,7 @@ static int cli(int argc,char **argv)
 				FSRefMakePath(&fsref, path, sizeof(path)) == noErr) {
 
 		} else if (getenv("SUDO_USER")) {
-			sprintf((char*)path, "/Users/%s/Desktop/", getenv("SUDO_USER"));
+			sprintf((char*)path, "/Users/%s/Desktop", getenv("SUDO_USER"));
 		} else {
 			fprintf(stdout, "%s", dump.str().c_str());
 			return 0;
