@@ -4,7 +4,7 @@
  * Use of this software is governed by the Business Source License included
  * in the LICENSE.TXT file in the project's root directory.
  *
- * Change Date: 2023-01-01
+ * Change Date: 2026-01-01
  *
  * On the date above, in accordance with the Business Source License, use
  * of this software will be governed by version 2.0 of the Apache License.
@@ -43,6 +43,8 @@
 #include <netinet/in_var.h>
 #include <netinet/icmp6.h>
 
+#include "MacDNSHelper.hpp"
+
 // OSX compile fix... in6_var defines this in a struct which namespaces it for C++ ... why?!?
 struct prf_ra {
 	u_char onlink : 1;
@@ -328,7 +330,7 @@ MacKextEthernetTap::MacKextEthernetTap(
 	Mutex::Lock _gl(globalTapCreateLock);
 
 	if (::stat("/dev/zt0",&stattmp)) {
-		long kextpid = (long)vfork();
+		long kextpid = (long)fork();
 		if (kextpid == 0) {
 			::chdir(homePath);
 			OSUtils::redirectUnixOutputs("/dev/null",(const char *)0);
@@ -403,7 +405,7 @@ MacKextEthernetTap::MacKextEthernetTap(
 	OSUtils::ztsnprintf(ethaddr,sizeof(ethaddr),"%.2x:%.2x:%.2x:%.2x:%.2x:%.2x",(int)mac[0],(int)mac[1],(int)mac[2],(int)mac[3],(int)mac[4],(int)mac[5]);
 	OSUtils::ztsnprintf(mtustr,sizeof(mtustr),"%u",_mtu);
 	OSUtils::ztsnprintf(metstr,sizeof(metstr),"%u",_metric);
-	long cpid = (long)vfork();
+	long cpid = (long)fork();
 	if (cpid == 0) {
 		::execl("/sbin/ifconfig","/sbin/ifconfig",_dev.c_str(),"lladdr",ethaddr,"mtu",mtustr,"metric",metstr,"up",(const char *)0);
 		::_exit(-1);
@@ -441,9 +443,13 @@ MacKextEthernetTap::MacKextEthernetTap(
 
 MacKextEthernetTap::~MacKextEthernetTap()
 {
+	MacDNSHelper::removeDNS(_nwid);
+
 	::write(_shutdownSignalPipe[1],"\0",1); // causes thread to exit
 	Thread::join(_thread);
-
+		for (std::thread &t : _rxThreads) {
+		t.join();
+	}
 	::close(_fd);
 	::close(_shutdownSignalPipe[0]);
 	::close(_shutdownSignalPipe[1]);
@@ -455,7 +461,7 @@ MacKextEthernetTap::~MacKextEthernetTap()
 
 			char tmp[16384];
 			sprintf(tmp,"%s/%s",_homePath.c_str(),"tap.kext");
-			long kextpid = (long)vfork();
+			long kextpid = (long)fork();
 			if (kextpid == 0) {
 				OSUtils::redirectUnixOutputs("/dev/null",(const char *)0);
 				::execl("/sbin/kextunload","/sbin/kextunload",tmp,(const char *)0);
@@ -484,7 +490,7 @@ bool MacKextEthernetTap::addIp(const InetAddress &ip)
 	if (!ip)
 		return false;
 
-	long cpid = (long)vfork();
+	long cpid = (long)fork();
 	if (cpid == 0) {
 		char tmp[128];
 		::execl("/sbin/ifconfig","/sbin/ifconfig",_dev.c_str(),(ip.ss_family == AF_INET6) ? "inet6" : "inet",ip.toString(tmp),"alias",(const char *)0);
@@ -505,7 +511,7 @@ bool MacKextEthernetTap::removeIp(const InetAddress &ip)
 	std::vector<InetAddress> allIps(ips());
 	for(std::vector<InetAddress>::iterator i(allIps.begin());i!=allIps.end();++i) {
 		if (*i == ip) {
-			long cpid = (long)vfork();
+			long cpid = (long)fork();
 			if (cpid == 0) {
 				char tmp[128];
 				execl("/sbin/ifconfig","/sbin/ifconfig",_dev.c_str(),(ip.ss_family == AF_INET6) ? "inet6" : "inet",ip.toIpString(tmp),"-alias",(const char *)0);
@@ -622,7 +628,7 @@ void MacKextEthernetTap::setMtu(unsigned int mtu)
 {
 	if (mtu != _mtu) {
 		_mtu = mtu;
-		long cpid = (long)vfork();
+		long cpid = (long)fork();
 		if (cpid == 0) {
 			char tmp[64];
 			OSUtils::ztsnprintf(tmp,sizeof(tmp),"%u",mtu);
@@ -687,4 +693,9 @@ void MacKextEthernetTap::threadMain()
 	}
 }
 
+void MacKextEthernetTap::setDns(const char *domain, const std::vector<InetAddress> &servers)
+{
+	MacDNSHelper::setDNS(_nwid, domain, servers);
+}
+
 } // namespace ZeroTier
